/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.esb.persistence.dao;

import org.wso2.esb.util.HibernateConfig;
import org.wso2.esb.persistence.dataobject.RegistryEntryDO;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.criterion.Expression;

public class RegistryEntryDAO extends BaseDAO {

    public RegistryEntryDAO(HibernateConfig hbConfig) {
        super(hbConfig);
    }

    public void addRegistryEntry(RegistryEntryDO registryEntryDO) {
        if (registryEntryDO != null && registryEntryDO.getRegistryKey() != null) {
            super.create(registryEntryDO);
        } else {
            throw new RuntimeException("Invalid registry entry for add " + registryEntryDO);
        }
    }

    public void updateRegistryEntry(RegistryEntryDO registryEntryDO) {
        if (registryEntryDO != null && registryEntryDO.getRegistryKey() != null) {
            RegistryEntryDO storedEntry = getRegistryEntry(registryEntryDO.getRegistryKey());
            if (storedEntry != null) {
                storedEntry.setExpiryTime(registryEntryDO.getExpiryTime());
                super.update(storedEntry);
            }
        } else {
            throw new RuntimeException("Invalid registry entry for update " + registryEntryDO);
        }
    }

    public void saveOrUpdateRegistryEntry(RegistryEntryDO registryEntryDO) {
        if (registryEntryDO != null && registryEntryDO.getRegistryKey() != null) {
            RegistryEntryDO storedEntry = getRegistryEntry(registryEntryDO.getRegistryKey());
            if (storedEntry != null) {
                storedEntry.setExpiryTime(registryEntryDO.getExpiryTime());
                super.update(storedEntry);
            } else {
                super.create(registryEntryDO);
            }
        } else {
            throw new RuntimeException("Invalid registry entry for save/update " + registryEntryDO);
        }
    }

    public RegistryEntryDO getRegistryEntry(String key) {

        Session session = hbConfig.currentSession();
        Transaction tx = null;
        RegistryEntryDO registryEntryDO = null;
        try {
            tx = session.beginTransaction();
            Criteria criteria = session.createCriteria(RegistryEntryDO.class);
            criteria.add(Expression.eq("registryKey", key.trim()));
            registryEntryDO = (RegistryEntryDO) criteria.uniqueResult();
            session.flush();
            tx.commit();
        } catch (HibernateException e) {
            tx.rollback();
            String msg = "Error getting registry key for key : " + key;
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {            
            hbConfig.closeSession();
        }

        return registryEntryDO;
    }

    public void deleteRegistryEntry(String key) {

        Session session = hbConfig.currentSession();
        Transaction tx = null;
        RegistryEntryDO registryEntryDO = null;
        try {
            tx = session.beginTransaction();
            Criteria criteria = session.createCriteria(RegistryEntryDO.class);
            criteria.add(Expression.eq("registryKey", key.trim()));
            registryEntryDO = (RegistryEntryDO) criteria.uniqueResult();
            
            if (registryEntryDO != null) {
                session.delete(registryEntryDO);
            }
            tx.commit();
        } catch (HibernateException e) {
            tx.rollback();
            String msg = "Error getting registry key for key : " + key;
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }
    }
}
