/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.services;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.synapse.SynapseException;
import org.apache.synapse.endpoints.*;
import org.apache.synapse.endpoints.dispatch.Dispatcher;
import org.apache.synapse.endpoints.dispatch.HttpSessionDispatcher;
import org.apache.synapse.endpoints.dispatch.SimpleClientSessionDispatcher;
import org.apache.synapse.endpoints.dispatch.SoapSessionDispatcher;
import org.apache.synapse.endpoints.utils.EndpointDefinition;
import org.apache.synapse.config.SynapseConfiguration;
import org.apache.synapse.config.xml.XMLConfigurationBuilder;
import org.apache.synapse.config.xml.endpoints.EndpointAbstractFactory;
import org.apache.synapse.config.xml.endpoints.EndpointFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.esb.services.tos.*;

import javax.xml.namespace.QName;
import java.util.*;
/*
 * 
 */

public class EndpointAdmin extends AbstractESBAdmin {

    private static final Log log = LogFactory.getLog(EndpointAdmin.class);

    public static int ADDRESS_EP = 0;
    public static int FAILOVER_EP = 2;
    public static int WSDL_EP = 1;
    public static int LOADBALANCE_EP = 3;

    /**
     * Gets the details about the endpoints as a EndpointData array
     *
     * @return Array of EndpointData representing the endpoints in the
     *         SynapseConfiguration
     * @throws AxisFault if an error occured while getting the data from
     *                   SynapseConfiguration
     */
    public EndpointMetaData[] endpointData() throws AxisFault {
        SynapseConfiguration synapseConfiguration = getSynapseConfiguration();
        Map namedEndpointMap = synapseConfiguration.getDefinedEndpoints();
        Collection namedEndpointCollection = namedEndpointMap.values();
        return listToEndpointMetaDatas((Endpoint[])
                namedEndpointCollection.toArray(new Endpoint[namedEndpointCollection.size()]));

    }

    /**
     * @param endpointName
     * @throws AxisFault
     */
    public void enableStatistics(String endpointName) throws AxisFault {
        try {
            Endpoint ep = getSynapseConfiguration().getEndpoint(endpointName.trim());
            if (ep instanceof AddressEndpoint) {
                ((AddressEndpoint) ep).getEndpoint()
                        .setStatisticsEnable(org.apache.synapse.Constants.STATISTICS_ON);
            } else if (ep instanceof WSDLEndpoint) {
                ((WSDLEndpoint) ep).getEndpointDefinition()
                        .setStatisticsEnable(org.apache.synapse.Constants.STATISTICS_ON);
            } else {
                handleFault(log, "Selected endpoint : " + endpointName +
                        " does not support statistics", null);
            }
            log.info("Statistics enabled on endpoint : " + endpointName);
        } catch (SynapseException syne) {
            handleFault(log,
                    "Error enabling statistics for the endpoint : " + endpointName, syne);
        }
    }

    /**
     * @param endpointName
     * @throws AxisFault
     */
    public void disableStatistics(String endpointName) throws AxisFault {
        try {
            Endpoint ep = getSynapseConfiguration().getEndpoint(endpointName.trim());
            if (ep instanceof AddressEndpoint) {
                ((AddressEndpoint) ep).getEndpoint()
                        .setStatisticsEnable(org.apache.synapse.Constants.STATISTICS_OFF);
            } else if (ep instanceof WSDLEndpoint) {
                ((WSDLEndpoint) ep).getEndpointDefinition()
                        .setStatisticsEnable(org.apache.synapse.Constants.STATISTICS_OFF);
            } else {
                handleFault(log, "Selected endpoint : " + endpointName +
                        " does not support statistics", null);
            }
            log.info("Statistics disabled on endpoint : " + endpointName);
        } catch (SynapseException syne) {
            handleFault(log,
                    "Error disabling statistics for the endpoint : " + endpointName, syne);
        }
    }

    public String[] getEndPointsNames() throws AxisFault {
        SynapseConfiguration synapseConfiguration = getSynapseConfiguration();
        Map namedEndpointMap = synapseConfiguration.getDefinedEndpoints();
        Collection namedEndpointCollection = namedEndpointMap.values();
        return listToNames((Endpoint[])
                namedEndpointCollection.toArray(new Endpoint[namedEndpointCollection.size()]));
    }

    private String[] listToNames(Endpoint[] eps) {
        if (eps == null) {
            return null;
        } else {
            String[] datas = new String[eps.length];
            for (int i = 0; i < eps.length; i++) {
                Endpoint ep = eps[i];
                datas[i] = ep.getName();
            }
            return datas;
        }
    }

    private EndpointMetaData[] listToEndpointMetaDatas(Endpoint[] eps) {
        if (eps == null) {
            return null;
        } else {
            EndpointMetaData[] datas = new EndpointMetaData[eps.length];
            for (int i = 0; i < eps.length; i++) {
                Endpoint ep = eps[i];
                if (ep instanceof AddressEndpoint) {
                    EndpointMetaData data = new EndpointMetaData();
                    data.setName(ep.getName());
                    data.setType(ADDRESS_EP);
                    EndpointDefinition def = ((AddressEndpoint) ep).getEndpoint();
                    if (def.getStatisticsEnable() == org.apache.synapse.Constants.STATISTICS_ON) {
                        data.setEnableStatistics(true);
                    } else {
                        data.setEnableStatistics(false);
                    }
                    datas[i] = data;
                }
                if (ep instanceof WSDLEndpoint) {
                    EndpointMetaData data = new EndpointMetaData();
                    data.setName(ep.getName());
                    data.setType(WSDL_EP);
                    EndpointDefinition def = ((WSDLEndpoint) ep).getEndpointDefinition();
                    if (def.getStatisticsEnable() == org.apache.synapse.Constants.STATISTICS_ON) {
                        data.setEnableStatistics(true);
                    } else {
                        data.setEnableStatistics(false);
                    }
                    datas[i] = data;
                } else if (ep instanceof FailoverEndpoint) {
                    EndpointMetaData data = new EndpointMetaData();
                    data.setName(ep.getName());
                    data.setType(FAILOVER_EP);
                    datas[i] = data;
                } else if (ep instanceof LoadbalanceEndpoint) {
                    EndpointMetaData data = new EndpointMetaData();
                    data.setName(ep.getName());
                    data.setType(LOADBALANCE_EP);
                    datas[i] = data;
                } else if (ep instanceof SALoadbalanceEndpoint) {
                    EndpointMetaData data = new EndpointMetaData();
                    data.setName(ep.getName());
                    data.setType(LOADBALANCE_EP);
                    datas[i] = data;
                }
            }
            return datas;
        }
    }

    private EndpointData getEndpointData(Endpoint ep) {
        if (ep == null) {
            return null;
        } else {
            if (ep instanceof AddressEndpoint) {
                EndpointDefinition def = ((AddressEndpoint) ep).getEndpoint();
                AddressEndpointData epData = new AddressEndpointData();
                epData.setSuspendDurationOnFailure(((AddressEndpoint) ep).getSuspendOnFailDuration());
                epData.setName(ep.getName());
                epData.setEndpointDefinition(def);
                return epData;
            }
            if (ep instanceof IndirectEndpoint) {
                IndirectEndpointData data = new IndirectEndpointData();
                data.setName(ep.getName());
                data.setKey(((IndirectEndpoint) ep).getKey());
                return data;
            }
            if (ep instanceof WSDLEndpoint) {
                WSDLEndpoint wsdlEndpoint = (WSDLEndpoint) ep;
                EndpointDefinition def = (wsdlEndpoint).getEndpointDefinition();
                WSDLEndpointData epData = new WSDLEndpointData();
                epData.setSuspendDurationOnFailure(wsdlEndpoint.getSuspendOnFailDuration());
                epData.setName(wsdlEndpoint.getName());
                epData.setPort(wsdlEndpoint.getPortName());
                epData.setService(wsdlEndpoint.getServiceName());
                epData.setUri(wsdlEndpoint.getWsdlURI());
                epData.setInLineWSDL(wsdlEndpoint.getWsdlDoc());
                epData.setEndpointDefinition(def);
                return epData;
            } else if (ep instanceof FailoverEndpoint) {
                FailOverEndpointData epData = new FailOverEndpointData();
                epData.setName(ep.getName());
                List list = ((FailoverEndpoint) ep).getEndpoints();
                epData.setEndpoints(listToEndpointDatas((Endpoint[]) list
                        .toArray(new Endpoint[list.size()])));
                return epData;
            } else if (ep instanceof LoadbalanceEndpoint) {
                LoadBalanceEndpointData epData = new LoadBalanceEndpointData();
                epData.setName(ep.getName());
                List list = ((LoadbalanceEndpoint) ep).getEndpoints();
                epData.setEndpoints(listToEndpointDatas((Endpoint[]) list
                        .toArray(new Endpoint[list.size()])));
                return epData;
            } else if (ep instanceof SALoadbalanceEndpoint) {
                LoadBalanceEndpointData epData = new LoadBalanceEndpointData();
                epData.setName(ep.getName());
                Dispatcher dispatcher = ((SALoadbalanceEndpoint) ep).getDispatcher();
                if (dispatcher instanceof HttpSessionDispatcher) {
                    epData.setSessiontype("http");
                } else if (dispatcher instanceof SimpleClientSessionDispatcher) {
                    epData.setSessiontype("simpleClientSession");
                } else if (dispatcher instanceof SoapSessionDispatcher) {
                    epData.setSessiontype("soap");
                }
                List list = ((SALoadbalanceEndpoint) ep).getEndpoints();
                epData.setEndpoints(listToEndpointDatas((Endpoint[]) list
                        .toArray(new Endpoint[list.size()])));
                return epData;
            } else {
                return null;
            }
        }
    }

    private EndpointData[] listToEndpointDatas(Endpoint[] eps) {
        if (eps == null) {
            return null;
        } else {
            EndpointData [] datas = new EndpointData[eps.length];
            for (int i = 0; i < eps.length; i++) {
                Endpoint ep = eps[i];
                if (ep instanceof AddressEndpoint) {
                    EndpointDefinition def = ((AddressEndpoint) ep).getEndpoint();
                    AddressEndpointData epData = new AddressEndpointData();
                    epData.setName(ep.getName());
                    epData.setSuspendDurationOnFailure(((AddressEndpoint) ep).getSuspendOnFailDuration());
                    epData.setEndpointDefinition(def);
                    datas[i] = epData;
                }
                if (ep instanceof IndirectEndpoint) {
                    IndirectEndpointData data = new IndirectEndpointData();
                    data.setName(ep.getName());
                    data.setKey(((IndirectEndpoint) ep).getKey());
                    datas[i] = data;
                }
                if (ep instanceof WSDLEndpoint) {
                    WSDLEndpoint wsdlEndpoint = (WSDLEndpoint) ep;
                    EndpointDefinition def = (wsdlEndpoint).getEndpointDefinition();
                    WSDLEndpointData epData = new WSDLEndpointData();
                    epData.setSuspendDurationOnFailure(wsdlEndpoint.getSuspendOnFailDuration());
                    epData.setName(wsdlEndpoint.getName());
                    epData.setPort(wsdlEndpoint.getPortName());
                    epData.setService(wsdlEndpoint.getServiceName());
                    epData.setUri(wsdlEndpoint.getWsdlURI());
                    epData.setInLineWSDL(wsdlEndpoint.getWsdlDoc());
                    epData.setEndpointDefinition(def);
                    datas[i] = epData;
                } else if (ep instanceof FailoverEndpoint) {
                    FailOverEndpointData epData = new FailOverEndpointData();
                    epData.setName(ep.getName());
                    List list = ((FailoverEndpoint) ep).getEndpoints();
                    epData.setEndpoints(listToEndpointDatas((Endpoint[]) list
                            .toArray(new Endpoint[list.size()])));
                    datas[i] = epData;
                } else if (ep instanceof LoadbalanceEndpoint) {
                    LoadBalanceEndpointData epData = new LoadBalanceEndpointData();
                    epData.setName(ep.getName());
                    List list = ((LoadbalanceEndpoint) ep).getEndpoints();
                    epData.setEndpoints(listToEndpointDatas((Endpoint[]) list
                            .toArray(new Endpoint[list.size()])));
                    datas[i] = epData;
                } else if (ep instanceof SALoadbalanceEndpoint) {
                    LoadBalanceEndpointData epData = new LoadBalanceEndpointData();
                    epData.setName(ep.getName());
                    Dispatcher dispatcher = ((SALoadbalanceEndpoint) ep).getDispatcher();
                    if (dispatcher instanceof HttpSessionDispatcher) {
                        epData.setSessiontype("http");
                    } else if (dispatcher instanceof SimpleClientSessionDispatcher) {
                        epData.setSessiontype("simpleClientSession");
                    } else if (dispatcher instanceof SoapSessionDispatcher) {
                        epData.setSessiontype("soap");
                    }
                    List list = ((SALoadbalanceEndpoint) ep).getEndpoints();
                    epData.setEndpoints(listToEndpointDatas((Endpoint[]) list
                            .toArray(new Endpoint[list.size()])));
                    datas[i] = epData;
                }
            }
            return datas;
        }
    }

    public EndpointData convertToEndpointData(OMElement epElement) throws AxisFault {
        EndpointFactory fac = EndpointAbstractFactory.getEndpointFactroy(epElement);
        if (fac != null) {
            Endpoint endpoint = fac.createEndpoint(epElement, true);
            if (endpoint != null) {
                return getEndpointData(endpoint);
            } else {
                handleFault(log, "Unable to create an endpoint definition", null);
            }
        } else {
            handleFault(log, "Unable to access the endpoint factory instance", null);
        }
        return null;
    }

    /**
     * Add an endpoint described by the given OMElement
     *
     * @param endpointElement - OMelement representing the endpoint that needs
     *                        to be added
     * @throws AxisFault if the element is not an endpoint or if an endpoint
     *                   wiht the same name exists
     */
    public boolean addEndpoint(OMElement endpointElement) throws AxisFault {

        if (endpointElement.getQName().getLocalPart()
                .equals(org.apache.synapse.config.xml.Constants.ENDPOINT_ELT.getLocalPart())) {

            String endpointName = endpointElement.getAttributeValue(new QName("name"));
            log.debug("Adding endpoint : " + endpointName + " to the configuration");

            if (endpointName != null && !("".equals(endpointName)) &&
                    getSynapseConfiguration().getLocalRegistry().get(endpointName.trim()) != null) {
                handleFault(log, "The name '" + endpointName +
                        "' is already used within the configuration", null);
            } else {
                XMLConfigurationBuilder.defineEndpoint(getSynapseConfiguration(),
                        endpointElement);
            }
            log.info("Added endpoint : " + endpointName + " to the configuration");
            return true;
        } else {
            handleFault(log, "Unable to create endpoint. Invalid XML definition", null);
        }
        return false;
    }

    /**
     * Save an existing endpoint from the given String representation of the XML
     *
     * @param endpointElement - String representing the XML which describes the
     *                        Endpoint element
     * @throws AxisFault if the endpoint does not exists in the
     *                   SynapseConfiguration
     */
    public boolean saveEndpoint(OMElement endpointElement) throws AxisFault {
        if (endpointElement.getQName().getLocalPart()
                .equals(org.apache.synapse.config.xml.Constants.ENDPOINT_ELT.getLocalPart())) {

            String endpointName = endpointElement.getAttributeValue(new QName("name"));
            log.info("Updating the definition of the endpoint : " + endpointName);

            if (endpointName != null &&
                    !("".equals(endpointName)) &&
                    getSynapseConfiguration().getEndpoint(endpointName.trim()) == null) {
                handleFault(log,
                        "The endpoint named '" + endpointName + "' does not exist", null);
            } else {
                Endpoint endpoint = EndpointAbstractFactory.getEndpointFactroy(
                        endpointElement).createEndpoint(endpointElement, false);
                getSynapseConfiguration().addEndpoint(endpointName.trim(), endpoint);
            }
            log.info("Updated the definition of the endpoint : " + endpointName);
            return true;
        } else {
            handleFault(log, "Unable to update endpoint. Invalid XML definition", null);
        }
        return false;
    }

    /**
     * Gets the endpoint element as an OMElement
     *
     * @param endpointName - name of the endpoint to be get
     * @return OMElement representing the endpoint with the given endpoint name
     */
    public EndpointData getEndpoint(String endpointName) throws AxisFault {
        SynapseConfiguration synapseConfiguration = getSynapseConfiguration();
        if (endpointName != null && !("".equals(endpointName)) && synapseConfiguration.getEndpoint(endpointName.trim()) != null)
        {
            return getEndpointData(synapseConfiguration.getEndpoint(endpointName.trim()));
        } else {
            handleFault(log, "The endpoint named '" + endpointName + "' does not exist", null);
        }
        return null;
    }

    /**
     * Deletes the endpoint from the SynapseConfiguration
     *
     * @param endpointName - name of the endpoint to be deleted
     * @throws AxisFault if the proxy service name given is not existent in the
     *                   synapse configuration
     */
    public boolean deleteEndpoint(String endpointName) throws AxisFault {

        if (endpointName != null && !("".equals(endpointName))) {
            log.info("Deleting endpoint : " + endpointName + " from the configuration");
            SynapseConfiguration synapseConfiguration
                    = getSynapseConfiguration();
            synapseConfiguration.removeEndpoint(endpointName.trim());
            log.info("Endpoint : " + endpointName + " removed from the configuration");
            return true;
        } else {
            log.warn("Invalid endpoint name for removal. Null or empty");
            return false;
        }
    }
}
