/*
* Copyright 2004,2005 The Apache Software Foundation.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.wso2.esb.statistics;

import org.apache.log4j.Logger;
import org.apache.synapse.core.SynapseEnvironment;
import org.apache.synapse.statistics.StatisticsCollector;
import org.apache.synapse.statistics.StatisticsHolder;
import org.wso2.esb.persistence.PersistenceManager;
import org.wso2.esb.statistics.persistence.StatisticsDBUtils;
import org.wso2.esb.util.HibernateConfigFactory;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 *
 *
 */
public class StatisticsReporterThread extends Thread {

    private static Logger log = Logger.getLogger(StatisticsReporterThread.class);
    private boolean shutdownRequested = false;

    PersistenceManager pm = new PersistenceManager(HibernateConfigFactory.getDefaultConfig(
        org.wso2.esb.Constants.WSO2ESB_HB_CONFIG_KEY, "wso2esb.hibernate.cfg.xml"));

    /** The reference to the synapse environment   */
    private SynapseEnvironment synapseEnvironment;

    /** IP Address of Server */
    private   String ipAddr;

    private long delay = 60*1000;

    /**
     * To construct it is need synapse Env. and PersistenceManager
     *
     * @param synapseEnvironment
     */
    public StatisticsReporterThread(SynapseEnvironment synapseEnvironment) {
        this.synapseEnvironment = synapseEnvironment;
    }

    public long getDelay() {
        return delay;
    }

    public void setDelay(long delay) {
        this.delay = delay;
    }

    /**
     *
     */
    public void run() {

        while (!shutdownRequested) {

            StatisticsCollector statisticsCollector = synapseEnvironment.getStatisticsCollector();

            // log.info("Statistics Reporter is running");
            if (statisticsCollector != null) {

                try {
                    if (ipAddr == null) {
                        InetAddress addr = InetAddress.getLocalHost();
                        if (addr != null) {
                            // Get IP Address
                            ipAddr = addr.getHostAddress();

                        }

                    }
                } catch (UnknownHostException e) {
                    log.warn("Unable to get hostname or IP address", e);
                }
                // report sequence statistics to database
                Map seqMap = new HashMap();
                seqMap.putAll(statisticsCollector.getSequenceStatistics());
                statisticsCollector.resetSequenceStatistics();
                for (Iterator sequenceStatistics = seqMap.values().iterator();
                     sequenceStatistics.hasNext();) {

                    Object statisticsHolderObject = sequenceStatistics.next();
                    if (statisticsHolderObject instanceof StatisticsHolder) {
                        StatisticsHolder statisticsHolder =
                            (StatisticsHolder) statisticsHolderObject;
                        StatisticsDBUtils.updateRecord(pm, ipAddr, statisticsHolder);
                        statisticsHolder.clearStatistics();
                    }
                }
                // report proxy service statistics to database
                Map proxyMap = new HashMap();
                proxyMap.putAll(statisticsCollector.getProxyServiceStatistics());
                statisticsCollector.resetProxyServiceStatistics();
                for (Iterator proxyServiceStatistics = proxyMap.values().iterator(); proxyServiceStatistics.hasNext();) {

                    Object statisticsHolderObject = proxyServiceStatistics.next();
                    if (statisticsHolderObject instanceof StatisticsHolder) {
                        StatisticsHolder statisticsHolder =
                            (StatisticsHolder) statisticsHolderObject;
                        StatisticsDBUtils.updateRecord(pm, ipAddr, statisticsHolder);
                        statisticsHolder.clearStatistics();
                    }
                }
                // report end point statistics to database
                Map epMap = new HashMap();
                epMap.putAll(statisticsCollector.getEndPointStatistics());
                statisticsCollector.resetEndPointStatistics();
                for (Iterator endPointStatistics = epMap.values().iterator();
                     endPointStatistics.hasNext();) {

                    Object statisticsHolderObject = endPointStatistics.next();
                    if (statisticsHolderObject instanceof StatisticsHolder) {
                        StatisticsHolder statisticsHolder =
                            (StatisticsHolder) statisticsHolderObject;
                        StatisticsDBUtils.updateRecord(pm, ipAddr, statisticsHolder);
                        statisticsHolder.clearStatistics();
                    }
                }
            }

            try {
                // sleep for one minute
                Thread.sleep(delay);   
            } catch (InterruptedException ignore) {}
        }
    }

    /**
     * 
     */
    public void shutdown() {
        log.info("Statistics reporter thread is being shutdown ");
        shutdownRequested = true;
    }
}
