/*
* Copyright 2004,2005 The Apache Software Foundation.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.wso2.esb.statistics.persistence;

import org.apache.synapse.statistics.Statistics;
import org.apache.synapse.statistics.StatisticsHolder;
import org.wso2.esb.Constants;
import org.wso2.esb.persistence.PersistenceManager;
import org.wso2.esb.persistence.dataobject.StatisticsDO;

import java.util.List;


/**
 * The Util class to access STATISTICS table
 */

public class StatisticsDBUtils {

    private static final String SERVER_ID_ = "serverId";
    private static final String CATEGORY_ = "category";
    private static final String NAME_ = "name";
    private static final String DIRECTION_ = "direction";

    private static StatisticsDO createStatisticsRecord(
            String serverId, int statisticsCategory,
            String key, int direction, Statistics statistics) {

        StatisticsDO statisticsDO = new StatisticsDO();
        statisticsDO.setName(key);
        statisticsDO.setDirection(direction);
        statisticsDO.setCategory(statisticsCategory);
        statisticsDO.setServerId(serverId);
        statisticsDO.setMaxTime(statistics.getMaxProcessingTime());
        statisticsDO.setMinTime(statistics.getMinProcessingTime());
        statisticsDO.setFaultCount(statistics.getFaultCount());
        statisticsDO.setTotalCount(statistics.getCount());
        statisticsDO.setAvgTime(statistics.getAvgProcessingTime());
        return statisticsDO;
    }

    /**
     * To insert a record to the statistics table
     *
     * @param pm
     * @param serverId
     * @param statisticsHolder
     * @
     */
    public static void insertRecord(PersistenceManager pm, String serverId,
                                    StatisticsHolder statisticsHolder) {

        String key = statisticsHolder.getKey();
        int statisticsCategory = statisticsHolder.getStatisticsCategory();
        if (statisticsHolder.getInFlowStatistics() != null) {
            pm.addStatisticsRecord(createStatisticsRecord(
                    serverId, statisticsCategory, key, 0, statisticsHolder.getInFlowStatistics()));
        }
        if (statisticsHolder.getOutFlowStatistics() != null) {
            pm.addStatisticsRecord(createStatisticsRecord(
                    serverId, statisticsCategory, key, 1, statisticsHolder.getOutFlowStatistics()));
        }
    }

    /**
     * To update a record of the statistics table
     *
     * @param pm
     * @param serverId
     * @param statisticsHolder
     * @
     */
    public static void updateRecord(PersistenceManager pm, String serverId,
                                    StatisticsHolder statisticsHolder) {

        String key = statisticsHolder.getKey();
        int statisticsCategory = statisticsHolder.getStatisticsCategory();
        if (statisticsHolder.getInFlowStatistics() != null) {
            updateRecord(pm, serverId, statisticsCategory,
                    key, 0, statisticsHolder.getInFlowStatistics());
        }
        if (statisticsHolder.getOutFlowStatistics() != null) {
            updateRecord(pm, serverId, statisticsCategory,
                    key, 1, statisticsHolder.getOutFlowStatistics());
        }
    }

    /**
     * A helper method to update a record of the statistics table
     *
     * @param pm
     * @param serverId
     * @param statisticsCategory
     * @param key
     * @param direction
     * @param statistics
     * @
     */
    private static void updateRecord(PersistenceManager pm, String serverId,
                                     int statisticsCategory, String key,
                                     int direction, Statistics statistics) {

        String queryString = "from " + Constants.DBAccess.STATISTICS_DO + " " +
                " where " + SERVER_ID_ + "= '" + serverId + "' and " +
                CATEGORY_ + "=" + statisticsCategory + " and " + NAME_ + "= '" + key +
                "' and " + DIRECTION_ + " = " + direction + "";

        int currentTotalCount = statistics.getCount();
        int currentFaultCount = statistics.getFaultCount();
        long currentMaxTime = statistics.getMaxProcessingTime();
        long currentMinTime = statistics.getMinProcessingTime();
        double currentAvgTime = statistics.getAvgProcessingTime();


        StatisticsDO[] resultSet;
        resultSet = pm.selectStatisticsRecords(queryString);
        int previousTotalCount, previousFaultCount;
        long previousMaxTime, previousMinTime;
        double previousAvgTime;

        StatisticsDO previousDO;
        if (resultSet != null && resultSet.length == 1) {
            previousDO = resultSet[0];
            previousTotalCount = (int) previousDO.getTotalCount();
            previousFaultCount = (int) previousDO.getFaultCount();
            previousMaxTime = previousDO.getMaxTime();
            previousMinTime = previousDO.getMinTime();
            previousAvgTime = previousDO.getAvgTime();
            int updatedTotalCount, updatedFaultCount;
            long updatedMaxTime, updatedMinTime;
            double updatedAvgTime;
            updatedTotalCount = (currentTotalCount + previousTotalCount);
            if (updatedTotalCount != 0) {
                updatedFaultCount = currentFaultCount + previousFaultCount;
                updatedAvgTime = (currentAvgTime * currentTotalCount +
                        previousAvgTime * previousTotalCount) / updatedTotalCount;
                if (currentMaxTime > previousMaxTime) {
                    updatedMaxTime = currentMaxTime;
                } else {
                    updatedMaxTime = previousMaxTime;
                }
                if (currentMinTime < previousMinTime) {
                    updatedMinTime = currentMinTime;
                } else {
                    updatedMinTime = previousMinTime;
                }
                previousDO.setFaultCount(updatedFaultCount);
                previousDO.setTotalCount(updatedTotalCount);
                previousDO.setMaxTime(updatedMaxTime);
                previousDO.setMinTime(updatedMinTime);
                previousDO.setAvgTime(updatedAvgTime);
                pm.updateStatisticsRecord(previousDO);
            }

        } else if (resultSet == null || resultSet.length == 0) {
            previousDO = createStatisticsRecord(
                    serverId, statisticsCategory, key, direction, statistics);
            pm.addStatisticsRecord(previousDO);
        }
    }


    public static StatisticsDO[] getStatistics(PersistenceManager pm, String serverid) {

        String queryString = "from " + Constants.DBAccess.STATISTICS_DO +
                " where " + SERVER_ID_ + " = '" + serverid + "'";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getStatistics(
            PersistenceManager pm, String serverid, int direction) {

        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO +
                " where " + SERVER_ID_ + " = '" + serverid +
                "' and " + DIRECTION_ + " = " + direction + "";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getStatistics(PersistenceManager pm, String serverid,
                                               int direction, int category) {

        String queryString = "from " + Constants.DBAccess.STATISTICS_DO + " " + " where " +
                SERVER_ID_ + " = '" + serverid + "' and " + CATEGORY_ + "= " + category + " and " +
                DIRECTION_ + " = " + direction + "";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getTotalStatisticsForServer(
            PersistenceManager pm, String serverid) {

        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " +
                " where " + SERVER_ID_ + " = '" + serverid + "'";
        return pm.selectStatisticsRecords(queryString);
    }


    public static StatisticsDO[] getStatistics(PersistenceManager pm, String serverid,
                                               int category, String name) {

        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " + " where " +
                SERVER_ID_ + " = '" + serverid + "' and " + CATEGORY_ + "= " + category + " and " +
                NAME_ + " = '" + name + "'";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO getStatistics(PersistenceManager pm, String serverid, int category,
                                             int direction, String name) {

        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " + " where " +
                SERVER_ID_ + " = '" + serverid + "' and " + CATEGORY_ + "= " + category + " and " +
                NAME_ + " = '" + name + "'" + " and " + DIRECTION_ + " = " + direction + "";
        return (StatisticsDO) pm.createAnewObjectFromStatistics(queryString);
    }

    public static StatisticsDO[] getStatistics(PersistenceManager pm, int category, String name) {
        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " +
                " where " + CATEGORY_ + "= " + category +
                " and " + NAME_ + " = '" + name + "'";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getStatistics(PersistenceManager pm, int category) {
        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " +
                " where " + CATEGORY_ + "= " + category +
                "";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getStatistics(PersistenceManager pm, int category, int direction) {
        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " + " where " +
                CATEGORY_ + "= " + category + " and " + DIRECTION_ + " = " + direction + " ";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getStatisticsForServer(PersistenceManager pm, String serverid,
                                                        int category) {

        String queryString = "from " + Constants.DBAccess.STATISTICS_DO + " " + " where " +
                SERVER_ID_ + " = '" + serverid + "' and " + CATEGORY_ + " = " + category + "";
        return pm.selectStatisticsRecords(queryString);
    }

    public static StatisticsDO[] getStatisticsForServer(PersistenceManager pm, String serverid,
                                                        int direction, int category) {

        String queryString = "from  " + Constants.DBAccess.STATISTICS_DO + " " + " where " +
                SERVER_ID_ + " = '" + serverid + "' and " + CATEGORY_ + "= " + category + " and " +
                DIRECTION_ + " = " + direction + "";
        return pm.selectStatisticsRecords(queryString);
    }

    public static Object getSummaryStatisticsForServer(PersistenceManager pm, String serverid,
                                                       int direction, int category) {

        String queryString = "select sum(totalCount), sum(faultCount), avg(maxTime), " +
                "avg(minTime), avg(avgTime) from  " + Constants.DBAccess.STATISTICS_DO + " " +
                " where " + SERVER_ID_ + " = '" + serverid + "' and " + CATEGORY_ + "= " +
                category + " and " + DIRECTION_ + " = " + direction + "";
        return pm.createAnewObjectFromStatistics(queryString);
    }

    public static List getTotalCountForServerS(PersistenceManager pm, int category) {
        String queryString = "select  " + SERVER_ID_ + " ,sum(totalCount) from  " +
                Constants.DBAccess.STATISTICS_DO + " " +
                " where " + CATEGORY_ + "= " + category + " group by " + SERVER_ID_ + " ";

        return pm.selectCustomObjectList(queryString);
    }

    public static Object getServersIds(PersistenceManager pm) {
        String queryString = "select distinct " + SERVER_ID_ + " from  " +
                Constants.DBAccess.STATISTICS_DO + " ";
        return pm.selectCustomObjectList(queryString);
    }

    public static Object getUniqueNames(PersistenceManager pm, int category) {
        String queryString = "select distinct " + NAME_ + " from  " +
                Constants.DBAccess.STATISTICS_DO + " where " + CATEGORY_ + "= " + category + "  ";
        return pm.selectCustomObjectList(queryString);
    }

    public static List getTotalCountForCategory(PersistenceManager pm, int category) {
        String queryString = "select " + NAME_ + ",sum(totalCount) from  " +
                Constants.DBAccess.STATISTICS_DO + " " +
                " where " + CATEGORY_ + "= " + category + " group by " + NAME_ + " ";
        return pm.selectCustomObjectList(queryString);
    }

   public static int clearAll(PersistenceManager pm){
     String queryString =  "DELETE  " + Constants.DBAccess.STATISTICS_DO;
     return pm.clearAll(queryString);
   }
}
