/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.esb.transport.jetty;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.transport.http.util.SOAPUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class Axis2SOAPHandler extends Axis2Handler {

    private static Log log = LogFactory.getLog(Axis2SOAPHandler.class);

    public Axis2SOAPHandler(ConfigurationContext configurationContext) {
        super(configurationContext);
    }

    protected boolean processPostRequest(MessageContext msgContext,
                                         HttpServletRequest request,
                                         HttpServletResponse response) throws AxisFault {
        try {
            return new SOAPUtil().processPostRequest(msgContext, request, response);
        } catch (AxisFault axisFault) {
            if (axisFault.getMessage().toLowerCase().indexOf("invalid service group id") == -1) {
                log.error(axisFault);
            }
            throw axisFault;
        }
    }

    protected boolean processGetRequest(MessageContext msgContext,
                                        HttpServletRequest request,
                                        HttpServletResponse response) throws AxisFault {
        try {
            return new HttpGetRequestProcessor(configurationContext).
                    processRequest(msgContext, request, response);
        } catch (AxisFault axisFault) {
            log.error(axisFault);
            throw axisFault;
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
    }
}
