/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.esb.transport.jetty;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMException;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.PolicyInclude;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.transport.http.HTTPTransportReceiver;
import org.apache.axis2.transport.http.HTTPTransportUtils;
import org.apache.axis2.util.ExternalPolicySerializer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyReference;
import org.apache.neethi.PolicyRegistry;
import org.apache.ws.commons.schema.XmlSchema;
import org.wso2.utils.NetworkUtils;
import org.wso2.utils.ServerConfiguration;
import org.wso2.utils.ServerException;
import org.wso2.esb.ServiceBusManager;
import org.wso2.esb.transport.ServiceHTMLProcessor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.stream.*;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;

public class HttpGetRequestProcessor {
    protected ConfigurationContext configurationContext;
    private Log log = LogFactory.getLog(HttpGetRequestProcessor.class);

    public HttpGetRequestProcessor(ConfigurationContext configurationContext) {
        this.configurationContext = configurationContext;
    }

    /**
     * Handle RSS Feeds
     *
     * @param msgContext
     * @param request
     * @param response
     * @return
     * @throws org.apache.axis2.AxisFault
     */
    public boolean processRequest(MessageContext msgContext,
                                  HttpServletRequest request,
                                  HttpServletResponse response)
            throws AxisFault, ServerException {
        try {
            String requestURI = request.getRequestURI();
            String url = request.getPathInfo();
            String soapAction =
                    request.getHeader(HTTPConstants.HEADER_SOAP_ACTION);

            int indexOfServletPath =
                    request.getContextPath().indexOf(configurationContext.getServiceContextPath());
            String contextRoot;
            if (indexOfServletPath == -1) {
                contextRoot = request.getContextPath();
            } else {
                contextRoot = requestURI.substring(0, indexOfServletPath);
            }

            /*if (requestURI.indexOf("rss") >= 0) {
                return processRssFeed(response, contextRoot);
            }
            if (requestURI.indexOf("atom") >= 0) {
                return processAtomFeed(response, contextRoot);
            }*/
            String queryString = request.getQueryString();
            if (queryString != null) {
                if (queryString.toLowerCase().indexOf("info") == 0) {
                    return processInfo(requestURI, response);
                } else if (queryString.toLowerCase().indexOf("xsd") == 0) {
                    return processXsd(requestURI, response, request);
                } else if (queryString.toLowerCase().indexOf("policy") == 0) {
                    return processPolicy(msgContext, requestURI, response, request);
                } else if (queryString.toLowerCase().indexOf("wsdl2") == 0) {
                    return printWSDL(request, response, 1);
                } else if (queryString.toLowerCase().indexOf("wsdl") == 0) {
                    return printWSDL(request, response, 0);
                } else if (queryString.toLowerCase().indexOf("cert") == 0) {
                    return processCert(response, request);
                }
            }
            boolean isProcessed =
                    HTTPTransportUtils.
                            processHTTPGetRequest(msgContext,
                                                  response.getOutputStream(),
                                                  soapAction,
                                                  request.getRequestURI(),
                                                  configurationContext,
                                                  HTTPTransportReceiver.
                                                          getGetRequestParameters(url));

            if (!isProcessed) {
                response.setStatus(200);
                response.setContentType("text/html");
                response.getOutputStream().print(ServiceHTMLProcessor.
                        getServicesHTML(configurationContext));
                return true;
            }
            return isProcessed;
        } catch (IOException ioException) {
            throw  AxisFault.makeFault(ioException);
        }
    }

    private boolean processCert(HttpServletResponse response,
                                HttpServletRequest request) throws IOException {

        String requestURI = request.getRequestURI();
        String serviceName = requestURI.substring(requestURI.lastIndexOf("/") + 1);

        AxisService axisService =
                configurationContext.getAxisConfiguration().getServiceForActivation(serviceName);
        OutputStream outputStream = response.getOutputStream();
        if (!axisService.isActive()) {
            response.setContentType("text/html");
            outputStream.write(("<b>Service " + serviceName +
                                " is inactive. Cannot retrieve certificate.</b>").getBytes());
            outputStream.flush();
        } else {
            response.setContentType("text/html");
            outputStream.write(("<b>Service " + serviceName +
                                " does not have a private key.</b>").getBytes());
            outputStream.flush();
        }

        return false;
    }

    /**
     * wsdlType = 0, wsdl1.1; wsdlType = 1 ; wsdl2.0
     *
     * @param request
     * @param response
     * @param wsdlType
     * @return boolean
     */
    private boolean printWSDL(HttpServletRequest request,
                              HttpServletResponse response,
                              int wsdlType) throws IOException {
        String requestURI = request.getRequestURI();
        String serviceName = requestURI.substring(requestURI.lastIndexOf("/") + 1);
        String requestURL = request.getRequestURL().toString();
        int ipindex = requestURL.indexOf("//");
        String ip = null;
        if (ipindex >= 0) {
            ip = requestURL.substring(ipindex + 2, requestURL.length());
            int separatorIndex = ip.indexOf(":");
            if (separatorIndex > 0) {
                ip = ip.substring(0, separatorIndex);
            }
        }
        AxisService axisService =
                configurationContext.getAxisConfiguration().getServiceForActivation(serviceName);

        OutputStream outputStream = response.getOutputStream();
        if (axisService != null) {
            if (!axisService.isActive()) {
                response.setContentType("text/html");
                outputStream.write(("<b>Service " +
                                    serviceName +
                                    " is inactive. Cannot display WSDL document.</b>").getBytes());
                outputStream.flush();
                return false;
            }

            response.setContentType("text/xml");
            // Filter out the 'root' from serviceContextPath
            String serviceContextPath = configurationContext.getServiceContextPath();
            int indexOfServletPath =
                    request.getContextPath().indexOf(configurationContext.getServiceContextPath());
            String contextRoot;
            if (indexOfServletPath == -1) {
                contextRoot = request.getContextPath();
            } else {
                contextRoot = requestURI.substring(0, indexOfServletPath);
            }

            int index = serviceContextPath.lastIndexOf("/");
            String servicePath = serviceContextPath.substring(index + 1,
                                                              serviceContextPath.length());
            if (wsdlType == 0) {
                ByteArrayOutputStream byteArryaOutStream = new ByteArrayOutputStream();

                axisService.printWSDL(byteArryaOutStream, ip, servicePath);
                writeSchema(byteArryaOutStream, outputStream, contextRoot,
                            "annotated-wsdl.xsl");

            } else if (wsdlType == 1) {
                axisService.printWSDL2(outputStream, ip, servicePath);
            } else {
                response.setContentType("text/xml");
                outputStream.write("<b>WSDL for service not found</b>".getBytes());
                outputStream.flush();
            }
        } else {
            response.setContentType("text/xml");
            outputStream.write(("<b>Service " + serviceName +
                                " not found. Cannot display WSDL document.</b>").getBytes());
            outputStream.flush();
            return false;
        }
        return true;
    }

    private boolean processPolicy(MessageContext msgContext,
                                  String requestURI,
                                  HttpServletResponse response,
                                  HttpServletRequest request) throws IOException {
        String serviceName = requestURI
                .substring(requestURI.lastIndexOf("/") + 1);
        AxisService axisService =
                configurationContext.getAxisConfiguration().getServiceForActivation(serviceName);
        OutputStream outputStream = response.getOutputStream();

        if (axisService != null) {
            if (!axisService.isActive()) {
                response.setContentType("text/html");
                outputStream.write(("<b>Service " +
                                    serviceName +
                                    " is inactive. Cannot display policies.</b>").getBytes());
                outputStream.flush();
                return false;
            }
            PolicyInclude policyInclude = axisService.getPolicyInclude();
            if (policyInclude == null) {
                response.setContentType("text/xml");
                outputStream.write("<b>Policy element is not found!</b>".getBytes());
                outputStream.flush();
                return true;

            }
            ArrayList policyElements = policyInclude.getPolicyElements();

            PolicyRegistry reg = policyInclude.getPolicyRegistry();

            ExternalPolicySerializer serializer = new ExternalPolicySerializer();
            if (msgContext.getConfigurationContext().getAxisConfiguration()
                    .getLocalPolicyAssertions() != null) {
                serializer.setAssertionsToFilter(msgContext.getConfigurationContext()
                        .getAxisConfiguration().getLocalPolicyAssertions());
            }


            if (policyElements.size() == 1) {
                response.setContentType("text/xml");
                if (policyElements.get(0) instanceof Policy) {
                    Policy policy = (Policy) policyElements.get(0);
                    serializer.serialize(policy, outputStream);
                } else if (policyElements.get(0) instanceof PolicyReference) {
                    String key = ((PolicyReference) policyElements.get(0)).getURI();
                    if (key.startsWith("#")) {
                        key = key.substring(key.indexOf("#") + 1);
                    }

                    Policy p = reg.lookup(key);
                    if (p == null) {
                        outputStream.write("<b>Policy element is not found!</b>".getBytes());
                    } else {
                        serializer.serialize(p, outputStream);
                    }
                }
            } else {
                // for many policies
                String idParam = request.getParameter("id");
                if (idParam != null) {
                    Object policyObject = policyElements.get(Integer.parseInt(idParam));
                    if (policyObject != null) {
                        response.setContentType("text/xml");
                        if (policyObject instanceof Policy) {
                            Policy policy = (Policy) policyObject;
                            serializer.serialize(policy, outputStream);
                        } else if (policyObject instanceof PolicyReference) {
                            String key = ((PolicyReference) policyObject).getURI();
                            if (key.startsWith("#")) {
                                key = key.substring(key.indexOf("#") + 1);
                            }
                            Policy p = reg.lookup(key);
                            if (p == null) {
                                outputStream
                                        .write("<b>Policy element is not found!</b>".getBytes());
                            } else {
                                serializer.serialize(p, outputStream);
                            }
                        }
                    } else {
                        outputStream.write("<b>Policy is not found!</b>".getBytes());
                    }
                } else {
                    if (policyElements.size() == 0) {
                        response.setContentType("text/xml");
                        outputStream.write("<b>Policy is not found!</b>".getBytes());
                    } else {
                        String ipAddress = "http://" + NetworkUtils.getLocalHostname() + ":" +
                                           ServiceBusManager.httpPort;
                        outputStream.write(("<html><head>" +
                                            "<title>WSO2 Web Services Application Server v" +
                                            ServerConfiguration.getInstance().getFirstProperty("Version") +
                                            " Management Console - " +
                                            axisService.getName() + " Service Policies</title>" +
                                            "</head>" +
                                            "<body>" +
                                            "<b>Policies for " + axisService.getName() +
                                            " service</b><br/><br/>").getBytes());
                        if (policyElements.size() != 0) {
                            for (int i = 0; i < policyElements.size(); i++) {
                                String st = "<a href=\"" + ipAddress +
                                            configurationContext.getServiceContextPath() + "/" +
                                            axisService.getName() + "?policy&id=" + i + "\">Policy " +
                                            i +
                                            "</a><br/>";
                                outputStream.write(st.getBytes());
                            }
                        } else {
                            outputStream.write("<p>No policies found</p>".getBytes());
                        }
                        outputStream.write("</body></html>".getBytes());
                    }

                }
            }
        } else { // Service is null
            response.setContentType("text/html");
            outputStream.write(("<b>Service " +
                                serviceName +
                                " not found. Cannot display policies.</b>").getBytes());
            outputStream.flush();
        }
        outputStream.flush();

        return true;
    }

    private boolean processXsd(String requestURI,
                               HttpServletResponse response,
                               HttpServletRequest request) throws IOException {
        String serviceName = requestURI
                .substring(requestURI.lastIndexOf("/") + 1);
        AxisService axisService =
                configurationContext.getAxisConfiguration().getServiceForActivation(serviceName);
        OutputStream outputStream = response.getOutputStream();
        if (axisService != null) {
            if (!axisService.isActive()) {
                response.setContentType("text/html");
                outputStream.write(("<b>Service " +
                                    serviceName +
                                    " is inactive. Cannot display Schema.</b>").getBytes());
                outputStream.flush();
                return false;
            }

            int indexOfServletPath =
                    request.getContextPath().indexOf(configurationContext.getServiceContextPath());
            String contextRoot;
            if (indexOfServletPath == -1) {
                contextRoot = request.getContextPath();
            } else {
                contextRoot = requestURI.substring(0, indexOfServletPath);
            }

            ArrayList schemas = axisService.getSchema();
            if (schemas.size() == 1) {

                response.setContentType("text/xml");

                // Write to the output stream
                processSchema((XmlSchema) schemas.get(0), outputStream, contextRoot);

            } else {
                String idParam = request.getParameter("id");
                if (idParam != null) {
                    XmlSchema schema = axisService.getSchema(Integer.parseInt(idParam));
                    if (schema != null) {
                        response.setContentType("text/xml");
                        processSchema(schema, outputStream, contextRoot);
                    } else {
                        outputStream.write("<b>Schema not found!</b>".getBytes());
                    }
                } else {
                    String ipAddress = "http://" + NetworkUtils.getLocalHostname() + ":" +
                                       ServiceBusManager.httpPort;
                    outputStream.write(("<html><head>" +
                                        "<title>WSO2 Web Services Application Server v" +
                                        ServerConfiguration.getInstance().getFirstProperty("Version") +
                                        "Management Console" +
                                        " - " + axisService.getName() + " Service Schema</title>" +
                                        "</head>" +
                                        "<body>" +
                                        "<b>Schemas for " + axisService.getName() +
                                        " service</b><br/><br/>").getBytes());
                    if (schemas.size() != 0) {
                        for (int i = 0; i < schemas.size(); i++) {
                            String st = "<a href=\"" + ipAddress +
                                        configurationContext.getServiceContextPath() + "/" +
                                        axisService.getName() + "?xsd&id=" + i + "\">Schema " + i +
                                        "</a><br/>";
                            outputStream.write(st.getBytes());
                        }
                    } else {
                        outputStream.write("<p>No schemas found</p>".getBytes());
                    }
                    outputStream.write("</body></html>".getBytes());
                }
            }
        } else {  // service is null
            response.setContentType("text/html");
            outputStream.write(("<b>Service " +
                                serviceName +
                                " is not found. Cannot display Schema.</b>").getBytes());
            outputStream.flush();
            return false;
        }
        return true;
    }


    private void processSchema(XmlSchema schema,
                               OutputStream outputStream,
                               String contextRoot) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        schema.write(baos);
        writeSchema(baos, outputStream, contextRoot, "annotated-xsd.xsl");
    }

    private void writeSchema(ByteArrayOutputStream byteArryaOutStream,
                             OutputStream out, String contextRoot, String annotatedXsl) {
        XMLStreamWriter writer;

        try {
            XMLStreamReader reader =
                    XMLInputFactory.newInstance().
                            createXMLStreamReader(new ByteArrayInputStream(byteArryaOutStream.toByteArray()));
            StAXOMBuilder builder = new StAXOMBuilder(reader);
            OMElement schemaDocument = builder.getDocumentElement();

            writer = XMLOutputFactory.newInstance().
                    createXMLStreamWriter(out);
            writer.writeProcessingInstruction("xml-stylesheet",
                                              "type=\"text/xsl\" href=\"" +
                                              contextRoot +
                                              "/styles/" + annotatedXsl + "\"");

            schemaDocument.serialize(writer);
            writer.flush();

        } catch (XMLStreamException e) {
            log.error(e);
        }

    }

    private boolean processInfo(String requestURI,
                                HttpServletResponse response) throws IOException {
        String serviceName = requestURI
                .substring(requestURI.lastIndexOf("/") + 1);
        response.setStatus(200);
        response.setContentType("text/html");

        response.getOutputStream().
                print(ServiceHTMLProcessor.printServiceHTML(serviceName, configurationContext));
        return true;
    }

    /*private boolean processAtomFeed(HttpServletResponse response,
                                    String contextRoot) {
        try {
            response.setContentType("text/xml; charset=utf-8");
            AtomFeed atomFeed =
                    (AtomFeed) System.getProperties().get(FeedConstants.ATOM_KEY);
            XMLStreamWriter writer =
                    XMLOutputFactory.newInstance().
                            createXMLStreamWriter(response.getOutputStream());
            writer.writeProcessingInstruction("xml-stylesheet",
                                              "type=\"text/xsl\" href=\"" +
                                              contextRoot +
                                              "/styles/atom.xsl\"");
            atomFeed.getFeedElement().serialize(writer);
            writer.flush();
        } catch (OMException e) {
            log.debug(e);
        } catch (Exception e) {
            log.error(e);
        }
        return true;
    }

    private boolean processRssFeed(HttpServletResponse response,
                                   String contextRoot) {
        try {
            response.setContentType("text/xml; charset=utf-8");
            RSSFeed rssFeed = (RSSFeed) System.getProperties().get(FeedConstants.RSS_KEY);
            XMLStreamWriter writer =
                    XMLOutputFactory.newInstance()
                            .createXMLStreamWriter(response.getOutputStream());
            writer.writeProcessingInstruction("xml-stylesheet",
                                              "type=\"text/xsl\" href=\"" +
                                              contextRoot +
                                              "/styles/rss.xsl\"");
            rssFeed.getFeedElement().serialize(writer);
            writer.flush();
        } catch (OMException e) {
            log.debug(e);
        } catch (Exception e) {
            log.error(e);
        }
        return true;
    }*/
}
