/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.transport.jetty;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.TransportInDescription;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.bio.SocketConnector;
import org.wso2.esb.ServiceBusManager;
import org.wso2.utils.NetworkUtils;
import org.wso2.utils.transport.AbstractTransportListener;
import org.wso2.utils.transport.http.HttpServerManager;

import javax.xml.namespace.QName;
import java.net.ServerSocket;

/**
 *
 *
 */

public class HttpTransportListener extends AbstractTransportListener {

    private static Log log = LogFactory.getLog(HttpTransportListener.class);

    public EndpointReference getEPRForService(String serviceName,
                                              String ip) throws AxisFault {
        return getEPR("http", serviceName, ip);
    }

    public EndpointReference[] getEPRsForService(String serviceName, String ip) throws AxisFault {
        try {
            ip = NetworkUtils.getLocalHostname();
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
        EndpointReference soapEPR = getEPR("http", serviceName, ip);
        return new EndpointReference[]{getEPRForService(serviceName, ip)};
    }

    public void destroy() {
        //TODO
    }

    public void start() throws AxisFault {
        //selectPort(configurationContext);
        Server httpServer = HttpServerManager.getServer(
                org.wso2.esb.transport.jetty.Constants.HTTP_SERVER);
        Connector connector = new SocketConnector();
        connector.setPort(port);
        ServiceBusManager.httpPort = port;
        httpServer.setConnectors(new Connector[]{connector});

        log.info("HTTP listener starting on port " + port);
    }

    private void selectPort(ConfigurationContext configCtx) {
        // check if configured port is available
        TransportInDescription trsIn = (TransportInDescription)
            configCtx.getAxisConfiguration().getTransportsIn().get(new QName("http"));

        if (trsIn != null) {

            port = Integer.parseInt(trsIn.getParameter("port").getValue().toString());

            while (true) {
                ServerSocket sock = null;
                try {
                    sock = new ServerSocket(port);
                    trsIn.getParameter("port").setValue(Integer.toString(port));
                    break;
                } catch (Exception e) {
                	System.out.println("Port "+port+" already in use. Trying alternate ports..");
                    if (port == 8080) {
                        port = 8008;
                    } else {
                        port++;
                    }
                } finally {
                    if (sock != null) {
                        try {
                            sock.close();
                        } catch (Exception e) {}
                    }
                }
            }
        }
    }
}
