package org.wso2.esb.transport.jetty;

import org.mortbay.jetty.handler.AbstractHandler;
import org.mortbay.jetty.Request;
import org.mortbay.jetty.HttpConnection;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.AxisFault;
import org.wso2.utils.transport.http.FaviconProvider;
import org.wso2.utils.NetworkUtils;
import org.wso2.esb.util.JettyWebConfiguration;
import org.wso2.esb.util.MIMEType2FileExtensionMap;
import org.wso2.esb.ServiceBusManager;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletException;
import java.util.Map;
import java.io.File;
import java.io.FileInputStream;
import java.io.EOFException;
import java.io.IOException;

public class ManagementConsoleRequestHandler extends AbstractHandler {

    protected static final Log log = LogFactory.getLog(ManagementConsoleRequestHandler.class);
    private static JettyWebConfiguration jettyConfig = JettyWebConfiguration.getInstance();

    private FaviconProvider faviconProvider;
    private ConfigurationContext configurationContext;

    public ManagementConsoleRequestHandler(ConfigurationContext configurationContext) {
        this.configurationContext = configurationContext;
        try {
            faviconProvider = new FaviconProvider();
            faviconProvider.setFavIconFromResource("org/wso2/esb/transport/jetty/favicon.ico");
        } catch (Exception e) {
            log.warn(e);
        }
    }

    private void generateWelcomePages(
            HttpServletRequest request, HttpServletResponse response) throws AxisFault {

        try {

            if (faviconProvider.provideFavIcon(request, response)) {
                return;
            }

            String requestURI = request.getRequestURI();
            String path = requestURI.equals(jettyConfig.adminWebCtx) ?
                          jettyConfig.adminWebResourceBase + "index.html" :
                          jettyConfig.adminWebResourceBase + requestURI;
            ServletOutputStream op = response.getOutputStream();
            if (requestURI.equals(jettyConfig.adminWebCtx)) {
                String fileContents = (String) ((Map) configurationContext.
                        getProperty(org.wso2.esb.Constants.GENERATED_PAGES)).get("index.html");
                if (fileContents != null) {
                    response.setContentType("text/html");
                    response.setContentLength(fileContents.getBytes().length);
                    op.write(fileContents.getBytes());
                } else {
                    op.write("index.html was not generated properly.".getBytes());
                }
                op.flush();
            } else {
                String urlKey;
                if (jettyConfig.adminWebCtx.equals("/")) {
                    urlKey = requestURI.substring(jettyConfig.adminWebCtx.length(),
                                                  requestURI.length());
                } else {
                    urlKey = requestURI.substring(1 + jettyConfig.adminWebCtx.length(),
                                                  requestURI.length() - 1);

                }
                String fileContents = (String) ((Map) configurationContext.
                        getProperty(org.wso2.esb.Constants.GENERATED_PAGES)).get(urlKey);
                if (fileContents != null) {
                    response.setContentType("text/html");
                    response.setContentLength(fileContents.getBytes().length);
                    op.write(fileContents.getBytes());
                    op.flush();
                } else {
                    File file = new File(path);
                    if (!file.exists()) {
                        op.write(("Requested resource " + requestURI + " not found").getBytes());
                        op.flush();
                        return;
                    }
                    if (!file.isDirectory()) {
                        FileInputStream fis = new FileInputStream(file);
                        byte[] fileBytes = new byte[(int) file.length()];
                        int length = fis.read(fileBytes);
                        if (length == 0) {
                            log.warn(file.getName() + " is 0 bytes");
                        }
                        response.setContentType(MIMEType2FileExtensionMap.getInstance().
                                getMIMEType(file));
                        response.setContentLength((int) file.length());
                        op.write(fileBytes);
                        fis.close();
                        op.close();
                    } else {
                        //TOdO Fix this
//                        op.write("Directory listing is forbidden".getBytes());
//                        return false;
                    }
                }
            }
        } catch (org.mortbay.jetty.EofException ignored) {
            // NO OP : Socket connection break. No need to handle this.
        } catch (EOFException ignored) {
            // NO OP : Socket connection break. No need to handle this.
        } catch (Exception e) {
            String msg = "Exception occurred while processing Request";
            log.error(msg, e);
            throw new AxisFault(msg, e);
        }
    }


    public void handle(String target, HttpServletRequest request, HttpServletResponse response,
                       int dispatch) throws IOException, ServletException {
        Request baseRequest = (request instanceof Request) ? (Request) request :
                              HttpConnection.getCurrentConnection().getRequest();
        baseRequest.setHandled(true);

        if (hostResolver(request, response)) {
            return;
        }

        generateWelcomePages(request,response);

    }

    /**
     * This will redirect the request first before doing anything else
     * @param request
     * @param response
     * @return boolean
     * @throws IOException
     */
    private boolean hostResolver(HttpServletRequest request,
                              HttpServletResponse response) throws IOException {
        String contextRoot = configurationContext.getContextRoot();
        if (request.getRequestURL().toString().indexOf(NetworkUtils.getLocalHostname()) == -1) {
            response.sendRedirect("https://" +
                                  NetworkUtils.getLocalHostname() + ":" + 9443 +
                                  (contextRoot.equals("/") ? contextRoot : contextRoot + "/"));
            return true;

        }
        return false;

    }
}
