package org.wso2.esb.util;

import org.apache.log4j.Logger;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;

import javax.xml.namespace.QName;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.io.File;

public class MIMEType2FileExtensionMap {
    public static String mappingFileName = "conf/mime-mappings.xml";

    private Map extensionToMimeMap;
    public static MIMEType2FileExtensionMap instance;
    private static Logger log = Logger.getLogger(MIMEType2FileExtensionMap.class);

    public static MIMEType2FileExtensionMap getInstance(){
        if(instance == null){
            instance = new MIMEType2FileExtensionMap();
        }
        return instance;
    }

    private MIMEType2FileExtensionMap() {
        extensionToMimeMap = new HashMap();
        init();
    }

    public String getMIMEType(File file) {
        String filename = file.getName();
        int index = filename.lastIndexOf(".");
        if (index == -1 || index == filename.length() - 1) {
            return "application/octet-stream";
        } else {
            String extension = filename.substring(index + 1);
            String contentType = (String) extensionToMimeMap.get(extension);
            return contentType != null ? contentType : "application/octet-stream";
        }
    }

    private void init() {

        try {
            OMElement doc = new StAXOMBuilder(mappingFileName).getDocumentElement();
            for (Iterator childIter = doc.getChildElements(); childIter.hasNext();) {
                OMElement mappingEle = (OMElement) childIter.next();
                String ext = mappingEle.getFirstChildWithName(new QName("Extension")).getText();
                String mimeType = mappingEle.getFirstChildWithName(new QName("MimeType")).getText();
                extensionToMimeMap.put(ext, mimeType);
            }
        } catch (Exception e) {
            String msg = "Unable to load all MIME Mappings";
            e.printStackTrace();
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        }
    }
}
